package com.mdtexfx.html;

import javafx.scene.layout.Region;
import javafx.scene.web.WebView;

import java.util.Arrays;

import static java.lang.String.format;

/**
 * Responsible for displaying an HTML document in a GUI.
 */
public class HtmlPreview extends Region implements HtmlRenderer {
  /**
   * The GUI to render the HTML as a web page.
   */
  private final WebView mView;

  private static final String HTML_PREFIX = """
    <!DOCTYPE html>
    <html lang='en'><head><title> </title><meta charset='utf-8'/></head>
    <body>
    """;
  private static final String HTML_SUFFIX = "</body></html>";
  private static final int HTML_PREFIX_LENGTH = HTML_PREFIX.length();

  private final StringBuilder mHtmlDocument = new StringBuilder( 65536 );

  public HtmlPreview() {
    // Retain the same prefix each time the HTML content is updated.
    mHtmlDocument.append( HTML_PREFIX );

    mView = new WebView();
    mView.prefWidthProperty().bind( widthProperty() );
    mView.prefHeightProperty().bind( heightProperty() );
    mView.getEngine()
         .setUserStyleSheetLocation(
           toDataUrl(
             "jlm_cmex10",
             "jlm_cmmi10",
             "jlm_cmr10",
             "jlm_cmsy10",
             "jlm_msam10",
             "jlm_msbm10"
           )
         );

    getChildren().add( mView );
  }

  @Override
  public String render( final String html ) {
    mHtmlDocument.setLength( HTML_PREFIX_LENGTH );
    mHtmlDocument.append( html );
    mHtmlDocument.append( HTML_SUFFIX );
    mView.getEngine().loadContent( mHtmlDocument.toString() );
    return html;
  }

  private String toDataUrl( final String... names ) {
    return format( "data:," + "%s".repeat( names.length ),
                   Arrays.stream( names ).map( this::toFontFace ).toArray() );
  }

  private String toFontFace( final String name ) {
    return format(
      "@font-face{font-family:'%s';src:url('%s') format('truetype');}",
      name, getResource( format( "/fonts/cm/%s.ttf", name ) ) );
  }

  private String getResource( final String path ) {
    return getClass().getResource( path ).toExternalForm();
  }
}
