package com.mdtexfx.processors.tex;

import com.mdtexfx.processors.ExecutorProcessor;
import com.mdtexfx.processors.Processor;
import com.whitemagicsoftware.tex.TeXFormula;
import com.whitemagicsoftware.tex.graphics.SvgGraphics2D;

import java.awt.*;
import java.util.regex.Pattern;

import static javax.swing.text.StyleConstants.ALIGN_CENTER;

public class TeXProcessor extends ExecutorProcessor<String> {
  private static final String REGEX = "\\$([^\s][^$]*[^\s])?\\$";
  private static final Pattern sPattern = Pattern.compile( REGEX );
  private static final Component sComponent = new Component() {};

  public TeXProcessor( final Processor<String> successor ) {
    super( successor );
  }

  private String toSvg( final String tex ) {
    final var len = tex.length();
    assert len > 3;

    final var formula = new TeXFormula( tex.substring( 1, len - 1 ) );
    final var icon = formula.createTeXIcon( ALIGN_CENTER, 20f );
    icon.setInsets( new Insets( 6, 0, 0, 4 ) );
    final var width = icon.getIconWidth();
    final var height = icon.getIconHeight();
    final var graphics = new SvgGraphics2D();
    graphics.initialize( width, height );
    icon.paintIcon( sComponent, graphics, 0, 0 );
    return graphics.toString();
  }

  @Override
  public String apply( String markdown ) {
    try {
      return sPattern.matcher( markdown ).replaceAll(
        ( result ) -> toSvg( result.group() ) );
    } catch( final Exception ex ) {
      // If any text breaks TeX, return the text without substitutions.
      return markdown;
    }
  }
}
