package com.mdtexfx.processors.tex;

import com.mdtexfx.processors.ProcessorContext;
import com.mdtexfx.processors.ProcessorFactory;
import org.junit.jupiter.api.Test;

import static com.mdtexfx.io.MediaType.TEXT_MARKDOWN;
import static com.mdtexfx.processors.ProcessorFactoryTest.HtmlCapture;
import static java.lang.System.nanoTime;
import static java.util.concurrent.TimeUnit.MINUTES;
import static java.util.concurrent.TimeUnit.NANOSECONDS;
import static org.junit.jupiter.api.Assertions.assertTrue;

class TeXProcessorTest {

  @Test
  void test_Formula_MarkdownInput_SvgOutput() {
    final var renderer = new HtmlCapture();
    final var context = new ProcessorContext( TEXT_MARKDOWN, renderer );
    final var processor = ProcessorFactory.create( context );
    final var html = processor.apply( "$e^{i\\pi} + 1 = 0$" );

    assertTrue( html.contains( "<svg" ) );
  }

  static boolean test_Performance_TeXInput_SvgOutput() {
    final var processor = new TeXProcessor( null );
    final var svg = processor.apply(
      """
        $\\frac{1}{c^2}\\frac{\\partial^2}{\\partial t^2}
        \\psi-\\nabla^2\\psi+\\frac{m^2c^2}{\\hbar^2}\\psi=0$
        """
    );

    return svg.startsWith( "<svg" );
  }

  @SuppressWarnings( "StatementWithEmptyBody" )
  public static void main( final String[] args ) {
    final long stop = nanoTime() + NANOSECONDS.convert( 5, MINUTES );
    while( nanoTime() < stop && test_Performance_TeXInput_SvgOutput() ) {
    }
  }
}
